﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Core;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Data;
using HeuristicLab.Common;
using HeuristicLab.Optimization;

namespace HeuristicLab.Problems.VehicleRouting.Encodings.Potvin {
  [Item("PotvinOnePointCrossoverMoveEvaluator", "Evaluates a customer relocation move for a VRP representation. ")]
  [StorableClass]
  public sealed class PotvinOnePointCrossoverMoveEvaluator : PotvinMoveEvaluator, IPotvinOnePointCrossoverMoveOperator {
    public ILookupParameter<PotvinOnePointCrossoverMove> OnePointCrossoverMoveParameter {
      get { return (ILookupParameter<PotvinOnePointCrossoverMove>)Parameters["PotvinOnePointCrossoverMove"]; }
    }

    public override ILookupParameter VRPMoveParameter {
      get { return OnePointCrossoverMoveParameter; }
    }
    [StorableConstructor]
    private PotvinOnePointCrossoverMoveEvaluator(bool deserializing) : base(deserializing) { }

    public PotvinOnePointCrossoverMoveEvaluator()
      : base() {
       Parameters.Add(new LookupParameter<PotvinOnePointCrossoverMove>("PotvinOnePointCrossoverMove", "The move that should be evaluated."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new PotvinOnePointCrossoverMoveEvaluator(this, cloner);
    }

    private PotvinOnePointCrossoverMoveEvaluator(PotvinOnePointCrossoverMoveEvaluator original, Cloner cloner)
      : base(original, cloner) {
    }

    protected override void EvaluateMove() {
      PotvinOnePointCrossoverMove move = OnePointCrossoverMoveParameter.ActualValue;

      PotvinEncoding newSolution = OnePointCrossoverMoveParameter.ActualValue.Individual.Clone() as PotvinEncoding;
      PotvinOnePointCrossoverMoveMaker.Apply(newSolution, move, ProblemInstance);

      UpdateEvaluation(newSolution);
    } 
  }
}
